//=========================================================
// LPC1114 Project
//=========================================================
// File Name : main.c
// Function  : Main Routine
//---------------------------------------------------------
// Rev.01 2011.02.26 Munetomo Maruyama
//---------------------------------------------------------
// Copyright (C) 2010-2011 Munetomo Maruyama
//=========================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include "array_com.h"
#include "color_led.h"
#include "gpio.h"
#include "oled.h"
#include "mems.h"
#include "systick.h"
#include "uart.h"

//========================
// Define Board Position
//========================
// XB
#define XB_PosX 0
#define XB_PosY 0

// OB
#define OB_PosX 0
#define OB_PosY 1

// Motor Driver
#define MD_PosX 0
#define MD_PosY 2

//===============================
// Mary Rover Body
//===============================
typedef struct
{
    uint32_t src_posx; // source node of token
    uint32_t src_posy; // source node of token
    uint32_t token_sn; // token serial number (unique)
    uint32_t dst_posx; // destination node of token
    uint32_t dst_posy; // destination node of token
    uint32_t systicks; // system ticks
    //
    int8_t   mems_x;
    int8_t   mems_y;
    int8_t   mems_z;
} sToken;

//---------------------------------------
// Motor Control Port
//---------------------------------------
// Motor Left  : IN1=PIO0_8, IN2=PIO0_9
//       Forward PIO0_8=H, PIO0_9=L
//       Back    PIO0_8=L, PIO0_9=H
//       Stop    PIO0_8=H, PIO0_9=H
// Motor Right : IN1=PIO1_9, IN2=PIO1_10
//       Forward PIO1_9=L, PIO1_10=H
//       Back    PIO1_9=H, PIO1_10=L
//       Stop    PIO1_9=H, PIO1_10=H
//---------------------
// Wheel: Right Forward
//---------------------
void Wheel_Right_Forward(void)
{
    // Right Forward
    GPIOSetDir  (1,  9, 1); // OUT
    GPIOSetValue(1,  9, 0); // LOW
    GPIOSetDir  (1, 10, 0); // HIZ
    GPIOSetValue(1, 10, 1); // HIGH
}
//---------------------
// Wheel: Right Back
//---------------------
void Wheel_Right_Back(void)
{
    // Right Back
    GPIOSetDir  (1,  9, 0); // HIZ
    GPIOSetValue(1,  9, 1); // HIGH
    GPIOSetDir  (1, 10, 1); // OUT
    GPIOSetValue(1, 10, 0); // LOW
}
//---------------------
// Wheel: Right Stop
//---------------------
void Wheel_Right_Stop(void)
{
    // Right Stop
    GPIOSetDir  (1,  9, 0); // HIZ
    GPIOSetValue(1,  9, 1); // HIGH
    GPIOSetDir  (1, 10, 0); // HIZ
    GPIOSetValue(1, 10, 1); // HIGH
}
//---------------------
// Wheel: Left Forward
//---------------------
void Wheel_Left_Forward(void)
{
    // Left Forward
    GPIOSetDir  (0,  8, 0); // HIZ
    GPIOSetValue(0,  8, 1); // HIGH
    GPIOSetDir  (0,  9, 1); // OUT
    GPIOSetValue(0,  9, 0); // LOW
}
//---------------------
// Wheel: Left Back
//---------------------
void Wheel_Left_Back(void)
{
    // Left Back
    GPIOSetDir  (0,  8, 1); // OUT
    GPIOSetValue(0,  8, 0); // LOW
    GPIOSetDir  (0,  9, 0); // HIZ
    GPIOSetValue(0,  9, 1); // HIGH
}
//---------------------
// Wheel: Left Stop
//---------------------
void Wheel_Left_Stop(void)
{
    // Left Stop
    GPIOSetDir  (0,  8, 0); // HIZ
    GPIOSetValue(0,  8, 1); // HIGH
    GPIOSetDir  (0,  9, 0); // HIZ
    GPIOSetValue(0,  9, 1); // HIGH
}

//-----------------------
// Main Routine
//-----------------------
int main(void)
{
    uint32_t x, y;
    uint32_t token_id[4][4];
    uint32_t token_sn;
    uint32_t port;
    uint32_t get_token;
    uint32_t count;
    sToken   sTOKEN;
    uint32_t initialized;
    //
    // Initialization
    //
    Init_SysTick();
    Init_Array_COM();
    Array_COM_ID_Assignment();
    Init_Color_LED();
    //
    initialized = 0;
    token_sn = 0;
    for (y = 0; y < 4; y++)
    {
        for (x = 0; x < 4; x++)
        {
            token_id[y][x] = 0;
        }
    }
    //
    // Forever Loop
    //
    while(1)
    {
        //
        // Draw Color LED
        //
        Draw_Color_LED();
        //
        // Get Token from Master Port, and if any, Bypass Token from Slave Port
        //
        get_token = 0;
        for (port = N; port <= W; port = port + (W - N))
        {
            if (Array_COM_Port_Open(port))
            {
                if ((count = Array_COM_Rx_Multi_Bytes(port, (uint8_t*)(&sTOKEN), 0)) > 0)
                {
                    //
                    // If the Token is the newest one?
                    //
                    if (sTOKEN.token_sn > token_id[sTOKEN.src_posy][sTOKEN.src_posx])
                    {
                        token_id[sTOKEN.src_posy][sTOKEN.src_posx] = sTOKEN.token_sn;
                        //
                        // Synchronize my Ticks (anyway)
                        //
                        Set_Ticks(sTOKEN.systicks);
                        //
                        // If destination of the Token is me?
                        //
                        if ((sTOKEN.dst_posx == Array_COM_Get_ID_PosX()) && (sTOKEN.dst_posy == Array_COM_Get_ID_PosY()))
                        {
                            get_token = 1;
                            break; // from the for loop
                        }
                        //
                        // If destination of the Token is NOT me?
                        //
                        else
                        {
                            // Send from both master ports.
                            if (Array_COM_Port_Open(E))
                            {
                                Array_COM_Tx_Multi_Bytes(E, (uint8_t*)(&sTOKEN), sizeof(sToken));
                            }
                            if (Array_COM_Port_Open(S))
                            {
                                Array_COM_Tx_Multi_Bytes(S, (uint8_t*)(&sTOKEN), sizeof(sToken));
                            }
                        }
                    }
                }
            }
        }
        //
        // I am XB
        //
        if ((Array_COM_Get_ID_PosX() == XB_PosX) && (Array_COM_Get_ID_PosY() == XB_PosY))
        {
            uint8_t rx_data;
            //
            // Initialize Hardware on 1st Time
            //
            if (initialized == 0)
            {
                // This routine is executed only once.
                initialized = 1;
                // Initialize
                UARTInit(9600);
            }
            //
            // Receive UART Data
            //
            while(1)
            {
                rx_data = UARTReceive_Byte();
                if (rx_data == 0x80) break;
            }
            sTOKEN.mems_x = (int8_t) UARTReceive_Byte();
            sTOKEN.mems_y = (int8_t) UARTReceive_Byte();
            sTOKEN.mems_z = (int8_t) UARTReceive_Byte();
            //
            // Make OLED Token
            //
            token_sn++;
            sTOKEN.src_posx = Array_COM_Get_ID_PosX();
            sTOKEN.src_posy = Array_COM_Get_ID_PosY();
            sTOKEN.token_sn = token_sn;
            sTOKEN.dst_posx = OB_PosX;
            sTOKEN.dst_posy = OB_PosY;
            sTOKEN.systicks = Get_Ticks();
            //
            // Send Token
            //
            if (Array_COM_Port_Open(E))
            {
                Array_COM_Tx_Multi_Bytes(E, (uint8_t*)(&sTOKEN), sizeof(sToken));
            }
            if (Array_COM_Port_Open(S))
            {
                Array_COM_Tx_Multi_Bytes(S, (uint8_t*)(&sTOKEN), sizeof(sToken));
            }
            //
            // Make Motor Token
            //
            token_sn++;
            sTOKEN.src_posx = Array_COM_Get_ID_PosX();
            sTOKEN.src_posy = Array_COM_Get_ID_PosY();
            sTOKEN.token_sn = token_sn;
            sTOKEN.dst_posx = MD_PosX;
            sTOKEN.dst_posy = MD_PosY;
            sTOKEN.systicks = Get_Ticks();
            //
            // Send Token
            //
            if (Array_COM_Port_Open(E))
            {
                Array_COM_Tx_Multi_Bytes(E, (uint8_t*)(&sTOKEN), sizeof(sToken));
            }
            if (Array_COM_Port_Open(S))
            {
                Array_COM_Tx_Multi_Bytes(S, (uint8_t*)(&sTOKEN), sizeof(sToken));
            }
        }
        //
        // I am OB.
        //
        if ((Array_COM_Get_ID_PosX() == OB_PosX) && (Array_COM_Get_ID_PosY() == OB_PosY))
        {
            //
            // Initialize Hardware on 1st Time
            //
            if (initialized == 0)
            {
                // This routine is executed only once.
                initialized = 1;
                // Initialize
                Init_OLED();
                OLED_Clear_Screen(OLED_BLK); // Clear Screen
                OLED_printf_Font(OLED_FONT_MEDIUM);
                OLED_printf_Position(0, 0);
                OLED_printf_Color(OLED_YEL, OLED_BLK);
                OLED_printf("MARY");
                OLED_printf_Font(OLED_FONT_SMALL);
                OLED_printf_Position(0, 2);
                OLED_printf_Color(OLED_GRN, OLED_BLK);
                OLED_printf("Rover Control");
            }
            //
            // Display MEMS Data on OLED
            //
            if (get_token)
            {
                OLED_printf_Font(OLED_FONT_MEDIUM);
                OLED_printf_Position(0, 3);
                OLED_printf_Color(OLED_RED, OLED_BLK);
                OLED_printf("X=%3d\n", (int32_t) sTOKEN.mems_x);
                OLED_printf_Position(0, 4);
                OLED_printf_Color(OLED_GRN, OLED_BLK);
                OLED_printf("Y=%3d\n", (int32_t) sTOKEN.mems_y);
                OLED_printf_Position(0, 5);
                OLED_printf_Color(OLED_BLU, OLED_BLK);
                OLED_printf("Z=%3d\n", (int32_t) sTOKEN.mems_z);
            }
        }
        //
        // I am Motor Driver.
        //
        if ((Array_COM_Get_ID_PosX() == MD_PosX) && (Array_COM_Get_ID_PosY() == MD_PosY))
        {
            //
            // Initialize Hardware on 1st Time
            //
            if (initialized == 0)
            {
                // This routine is executed only once.
                initialized = 1;
                // Initialize
                // Motor Drive Output (So far, Fixed to High Level)
                LPC_IOCON->PIO0_8  = 0x00; // GPIO
                LPC_IOCON->PIO0_9  = 0x00; // GPIO
                LPC_IOCON->PIO1_9  = 0x00; // GPIO
                LPC_IOCON->PIO1_10 = 0xc0; // GPIO
                //
                Wheel_Right_Stop();
                Wheel_Left_Stop();
            }
            //
            // Set Motor Wave
            //
            if (get_token)
            {

                // Turn Left
                if (sTOKEN.mems_x > 10)
                {
                    Wheel_Right_Forward();
                    Wheel_Left_Back();
                }
                // Turn Right
                else if (sTOKEN.mems_x < -10)
                {
                    Wheel_Right_Back();
                    Wheel_Left_Forward();
                }
                // Forward
                else if (sTOKEN.mems_y < -10)
                {
                    Wheel_Right_Forward();
                    Wheel_Left_Forward();
                }
                // Back
                else if (sTOKEN.mems_y > 10)
                {
                    Wheel_Right_Back();
                    Wheel_Left_Back();
                }
                // Stop
                else
                {
                    Wheel_Right_Stop();
                    Wheel_Left_Stop();
                }
            }
        }
    } // while(1)
    return 0;
}

//=========================================================
// End of Program
//=========================================================
